/*----------------------------------------------------------
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

The original author of this program is Hin Jang
March 20, 2003
-----------------------------------------------------------*/

// ----------------------------------------------------------
// catmullClark.h (for quad meshes only)
// ----------------------------------------------------------

#ifndef __CATMULL_CLARK_H__
#define __CATMULL_CLARK_H__

#include <string>
#include <fstream>
#include <map>
#include "kdTreeRoot.h"
#include "parser.h"

class CatmullClark
{

public:

  CatmullClark(KDTreeRoot *);
  ~CatmullClark();
  
  int load(char *);
  void draw(int);

private:

  void applySubdivisionRules(int);
  int findDuplicateVertex(Vertex *);
  int findAdjacentFaceID(vector<int>, int, int);

  vector<int> faceMask(vector<int>, int);
  vector<int> edgeMask(vector<int>, vector<int>,
		       vector<int>, int, int);
  int vertexMask(vector<int>, vector<int>, int, int);

  vector<Vertex*>  vertexPool;   // vertices of the surface
  vector<Polygon*> polygonPool;  // polygons of the surface
  map<int, int> adjacentEdgePoints;

  KDTreeRoot *forest;            // root of all forests

};

#endif // __CATMULL_CLARK_H__
