/*----------------------------------------------------------
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

The original author of this program is Hin Jang
March 20, 2003
-----------------------------------------------------------*/

// ----------------------------------------------------------
// kdTreeRoot.h (abstract class for all strategies)
// ----------------------------------------------------------

#ifndef __KDTREE_ROOT_H__
#define __KDTREE_ROOT_H__
#include "vertex.h"
#include "polygon.h"
#include <vector>

class KDTreeRoot
{

public:

  virtual void add(int, Polygon *) = 0;
  virtual void traverse(int) = 0;
  virtual void create(void) = 0;

  virtual void augmentVertexPool(Vertex *) = 0;
  virtual void augmentPolygonPool(Polygon *) = 0;

  virtual vector<Vertex*>  getVertexPool(void) = 0;
  virtual vector<Polygon*> getPolygonPool(void) = 0;

  virtual void setVertexPool(vector<Vertex*>) = 0;
  virtual void setPolygonPool(vector<Polygon*>) = 0;

  virtual int getMaxSubLevel(void) = 0;
  virtual void setMaxSubLevel(int) = 0;

  ~KDTreeRoot();   // should be virutal ;-)

protected:

  vector<Vertex*>  vertexPool;   // vertices of the surface
  vector<Polygon*> polygonPool;  // polygons of the surface
  int maxSubLevel;

  KDTreeRoot();

};

#endif // __KDTREE_ROOT_H__
