/*----------------------------------------------------------
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

The original author of this program is Hin Jang
March 20, 2003
-----------------------------------------------------------*/

// ----------------------------------------------------------
// quadTree.h
// ----------------------------------------------------------

#ifndef __QUAD_TREE_H__
#define __QUAD_TREE_H__
#include <iostream>
#include "vertex.h"
#include "polygon.h"

class QuadTree
{

public:

  QuadTree();
  ~QuadTree();

  void setParent(QuadTree *);
  void setChild(int, QuadTree *);

  void setPoly(Polygon *);
  void setSubdivisionLevel(int);

  QuadTree* getParent(void);
  QuadTree* getChild(int);

  Polygon* getPoly(void);
  int getSubdivisionLevel(void);

private:

  QuadTree *p;      // parent
  QuadTree *c[4];   // children

  Polygon *polygon; // a portion of the surface
  int subLevel;     // subdivision level at which this
                    // tree node exists

};

inline void QuadTree::setParent(QuadTree *parent)
{

  p = parent;

}

inline void QuadTree::setChild(int i, QuadTree *child)
{

  c[i] = child;

}

inline void QuadTree::setSubdivisionLevel(int i)
{

  subLevel = i;

}

inline QuadTree* QuadTree::getParent()
{

  return p;

}

inline QuadTree* QuadTree::getChild(int i)
{

  return c[i];

}

inline Polygon* QuadTree::getPoly()
{

  return polygon;

}

inline int QuadTree::getSubdivisionLevel()
{

  return subLevel;

}

#endif // __QUAD_TREE_H__

