/*----------------------------------------------------------
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

The original author of this program is Hin Jang
March 20, 2003
-----------------------------------------------------------*/

// ----------------------------------------------------------
// quadTreeRoot.h
// ----------------------------------------------------------

#ifndef __QUAD_TREE_ROOT_H__
#define __QUAD_TREE_ROOT_H__
#include "quadTree.h"
#include "kdTreeRoot.h"
#include <queue>

class QuadTreeRoot : public KDTreeRoot
{

public:

  QuadTreeRoot();
  ~QuadTreeRoot();  // should be virtual ;-)

  virtual void add(int, Polygon *);
  virtual void traverse(int);
  virtual void create(void);

  virtual void augmentVertexPool(Vertex *);
  virtual void augmentPolygonPool(Polygon *);

  virtual int getMaxSubLevel(void);
  virtual void setMaxSubLevel(int);

  vector<Vertex*>  getVertexPool(void);
  vector<Polygon*> getPolygonPool(void);

  void setVertexPool(vector<Vertex*>);
  void setPolygonPool(vector<Polygon*>);

private:

  void deepAdd(int *, int, int, Polygon *, QuadTree *);
  void createForest(QuadTreeRoot *, int, int);
  void findForest(QuadTreeRoot *, QuadTreeRoot **, int);
  void visitForest(int, QuadTreeRoot *);
  void visitNode(int, QuadTree *);

  QuadTreeRoot *n[4];   // neighbours
  QuadTree *root;       // actual tree
  int id;               // id of this forest node

};

inline void QuadTreeRoot::augmentVertexPool(Vertex *v)
{

  vertexPool.push_back(v);

}

inline void QuadTreeRoot::augmentPolygonPool(Polygon *p)
{

  polygonPool.push_back(p);

}

inline
vector<Vertex*> QuadTreeRoot::getVertexPool()
{

  return vertexPool;

}

inline
vector<Polygon*> QuadTreeRoot::getPolygonPool()
{

  return polygonPool;

}

inline
void QuadTreeRoot::setVertexPool(vector<Vertex*> pool)
{

  vertexPool = pool;

}

inline
void QuadTreeRoot::setPolygonPool(vector<Polygon*> pool)
{

  polygonPool = pool;

}

inline int QuadTreeRoot::getMaxSubLevel()
{

  return maxSubLevel;

}

inline void QuadTreeRoot::setMaxSubLevel(int s)
{

  maxSubLevel = s;

}

#endif // __QUAD_TREE_ROOT_H__
